#!/usr/bin/env bash

# bpgallery.sh - builds a simple 'gallery' from a collection of images
# Copyright (C) 2004 Brett Parker <iDunno@sommitrealweird.co.uk>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

set -f

VERSION="0.9.3+arch"

function bpgallery_default_head() {
cat <<END
<?xml version="1.0"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
	<title>${TITLE}</title>
	<link rel="stylesheet" href="style.css" type="text/css" />
</head>
<body>
END
}

function bpgallery_default_description() {
cat <<END
	<h1>${TITLE}</h1>
	${DESCRIPTION}
END
}

function bpgallery_default_thumbsline() {
cat << END
<div class="thumbnail"><a href="$filename"><img src="icons/$filename" /></a><div class="caption">$caption</div></div>
END
}

function bpgallery_default_tail() {
cat << END
</body>
</html>
END
}

function bpgallery_default_stylesheet() {
cat <<END
body {
        background: white;
        color: black;
        font-family: sans-serif;
        font-size: 10pt;
}

div.thumbnail {
        float: left;
        padding: 20px;
        border: 0px;
        width: ${WIDTH}px;
        height: ${MAXHEIGHT}px;
}

div.caption {
	width: 100%;
	text-align: center;
	font-weight: bold;
}

a {
        border: 0px;
}

img {
        border: 0px;
}
END
}

if [[ -e /usr/local/etc/bpgallery/config ]] ; then
	. /usr/local/etc/bpgallery/config
fi

if [[ -e /etc/bpgallery/config ]] ; then
	. /etc/bpgallery/config
fi

if [[ -e $HOME/.bpgallery.rc ]]; then
	. $HOME/.bpgallery.rc
fi

if [[ -z ${BPGALLERY_THEME} ]]; then
	BPGALLERY_THEME=default
fi

if [[ -e $HOME/.bpgallery.themes/${BPGALLERY_THEME} ]]; then
	. $HOME/.bpgallery.themes/${BPGALLERY_THEME}
elif [[ -e /usr/local/etc/bpgallery/themes/${BPGALLERY_THEME} ]]; then
	. /usr/local/etc/bpgallery/themes/${BPGALLERY_THEME}
elif [[ -e /etc/bpgallery/themes/${BPGALLERY_THEME} ]]; then
	. /etc/bpgallery/themes/${BPGALLERY_THEME}
fi

if [[ -z $TITLE ]]; then 
	TITLE="Photo Gallery"
fi

if [[ -z $CONVERTTOOL ]]; then
	CONVERTTOOL=convert
fi

if [[ -z $FINDCOMMAND ]]; then
	FINDCOMMAND=find
fi

if [[ -z $XARGSCOMMAND ]]; then
	XARGSCOMMAND=xargs
fi

if [[ -z $ECHOCOMMAND ]]; then
	ECHOCOMMAND=echo
fi

if [[ -z $SORTCOMMAND ]]; then
	SORTCOMMAND=sort
fi

if [[ -z $IDENTIFYCOMMAND ]]; then
	IDENTIFYCOMMAND=identify
fi

if [[ -z $HEADCOMMAND ]]; then
	HEADCOMMAND=head
fi

if [[ -z $GREPCOMMAND ]]; then
	GREPCOMMAND=grep
fi

if [[ -z $SEDCOMMAND ]]; then
	SEDCOMMAND=sed
fi

if [[ -z $WIDTH ]]; then
	WIDTH=100
fi

if [[ -z $IMAGEEXTENSIONS ]]; then
	IMAGEEXTENSIONS="jpeg jpg gif png";
fi

if [[ -z $WCCOMMAND ]]; then
	WCCOMMAND="wc -l"
fi

if [[ -z $CAPTIONHEIGHT ]]; then
	CAPTIONHEIGHT=75
fi

if [[ -z $DESCRIPTION ]]; then
	DESCRIPTION=""
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_head" > /dev/null ; then
	BPGALLERY_HEAD_FUNCTION="bpgallery_${BPGALLERY_THEME}_head"
else
	BPGALLERY_HEAD_FUNCTION="bpgallery_default_head"
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_description" > /dev/null ; then
	BPGALLERY_DESCRIPTION_FUNCTION="bpgallery_${BPGALLERY_THEME}_description"
else
	BPGALLERY_DESCRIPTION_FUNCTION="bpgallery_default_description"
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_thumbsline" > /dev/null ; then
	BPGALLERY_THUMBSLINE_FUNCTION="bpgallery_${BPGALLERY_THEME}_thumbsline"
else
	BPGALLERY_THUMBSLINE_FUNCTION="bpgallery_default_thumbsline"
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_tail" > /dev/null ; then
	BPGALLERY_TAIL_FUNCTION="bpgallery_${BPGALLERY_THEME}_tail"
else
	BPGALLERY_TAIL_FUNCTION="bpgallery_default_tail"
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_stylesheet" > /dev/null ; then
	BPGALLERY_STYLESHEET_FUNCTION="bpgallery_${BPGALLERY_THEME}_stylesheet"
else
	BPGALLERY_STYLESHEET_FUNCTION="bpgallery_default_stylesheet"
fi

FINDIMAGESOPTIONS=""

for imageext in $IMAGEEXTENSIONS; do
	FINDIMAGESOPTIONS=$FINDIMAGESOPTIONS' -o -type f -maxdepth 1 -iname '*.$imageext' -print0'
done

FINDIMAGESOPTIONS=${FINDIMAGESOPTIONS## -o }

function usage() {
cat <<END
Usage:
  $0 [--help|--version|<path to images>]

    --help
        displays this help screen
    --version
        displays the version and exits
	
END
}

function version() {
cat <<END
Version: $VERSION
END
}

if [[ -z $1 ]]; then
	echo "No path given"
	usage
	exit 1
fi

if [[ "$1" == "--help" || "$1" == "-h" ]]; then
	usage
	exit 0
fi

if [[ "$1" == "--version" || "$1" == "-v" ]]; then
	version
	exit 0
fi

if [[ ! -d $1 ]]; then
	echo "$1 is not a directory"
	usage
	exit 2
fi

cd "$1"

if ( ! $FINDCOMMAND . $FINDIMAGESOPTIONS > /dev/null 2>/dev/null ); then
	echo "$1 does not contain any images. Quitting."
	exit 4
fi

if [ ! -d icons ]; then
	mkdir icons

	totalimages=$($FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --replace echo {} | $WCCOMMAND);
	currentimage=0
	
	$FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --verbose --max-procs=4 --replace $CONVERTTOOL -resize $WIDTH '{}' 'icons/{}' 2>&1 | while read throwout; do $ECHOCOMMAND done: $currentimage/$totalimages images; currentimage=$((currentimage+1)); done
	
	echo done: $totalimages/$totalimages images
else
	echo "$1 already contains an icons folder, updating."

	imagestoupdate=$($FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --replace $ECHOCOMMAND {} | while read filename; do if [ ! -r "icons/$filename" ] || [ "$filename" -nt "icons/$filename" ]; then $ECHOCOMMAND $filename; fi; done | $WCCOMMAND)

	echo "images to update: $imagestoupdate"
	
	
	currentimage=0

	$FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --replace $ECHOCOMMAND {} | while read filename; do if [ ! -r "icons/$filename" ] || [ "$filename" -nt "icons/$filename" ] ; then $ECHOCOMMAND $filename; fi; done | $XARGSCOMMAND --verbose --max-procs=4 --replace $CONVERTTOOL -resize $WIDTH '{}' 'icons/{}' 2>&1 | while read throwout; do $ECHOCOMMAND done: $currentimage/$imagestoupdate images; currentimage=$((currentimage+1)); done
	echo done: $imagestoupdate/$imagestoupdate images
fi

if [ -r description.txt ] ; then
	DESCRIPTION=$($SEDCOMMAND -e '1 { s/^/<p>/; }; /^$/ { s,$,</p><p>,; }; $ { s,$,</p>, };' description.txt)
else
	DESCRIPTION=""
fi


$BPGALLERY_HEAD_FUNCTION > index.html
$BPGALLERY_DESCRIPTION_FUNCTION >> index.html

$FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --replace $ECHOCOMMAND {} |$SORTCOMMAND -g | while read filename; do filename=${filename#./}; if [ -r captions.txt ]; then caption=$($GREPCOMMAND -E "^$filename	" captions.txt); caption=${caption#*	}; else caption=""; fi; $BPGALLERY_THUMBSLINE_FUNCTION; done >> index.html

$BPGALLERY_TAIL_FUNCTION >> index.html

cd icons

MAXHEIGHT=$($FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 $IDENTIFYCOMMAND -format "%h\n" | $GREPCOMMAND -v "^$" | $SORTCOMMAND -g -r | $HEADCOMMAND -n 1)
MAXWIDTH=$($FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 $IDENTIFYCOMMAND -format "%w\n" | $GREPCOMMAND -v "^$" | $SORTCOMMAND -g -r | $HEADCOMMAND -n 1)

cd ..

# add a bit to the maxheight for the size of the caption
MAXHEIGHT=$((MAXHEIGHT+$CAPTIONHEIGHT))

$BPGALLERY_STYLESHEET_FUNCTION > style.css

